#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function, unicode_literals)

import sys
import argparse
import os
import re
import imp
import shutil
import subprocess

from envy import VERSION
from envy.decorators import validate_pkg, validate_env

def get_envy_base():
    return os.path.expanduser("~/.envies")


def get_active_venv():
    if 'VIRTUAL_ENV' in os.environ:
        return os.environ['VIRTUAL_ENV'].split('/')[-1]

    if 'WORKON_HOME' in os.environ:
        venv_root = os.environ['WORKON_HOME'].split('/')[-1]
        return re.search("{}/([^/]{{1,}})/bin".format(venv_root), sys.prefix).group(1)

    return re.search(".virtualenvs/([^/]{1,})/bin", sys.prefix).group(1)


def get_package_name(pkg_path):
    if ('/' not in pkg_path and pkg_path.endswith('.py')) or pkg_path == '.':
        return os.getcwd().split("/")[-1]

    return pkg_path.split('/')[0]


def get_envy_path(pkg_path):
    return os.path.expanduser("~/.envies/{}/{}".format(get_active_venv(), get_package_name(pkg_path)))


def get_venv_full_package_path(pkg_path):
    return imp.find_module(get_package_name(pkg_path))[1]


def original_backed_up(pkg_path):
    if not os.path.isdir(get_envy_base()):
        os.makedirs('{}'.format(get_envy_base()))

    if not os.path.isdir(get_envy_base() + "/{}".format(get_active_venv())):
        os.makedirs('{}'.format(get_envy_base() +  "/{}".format(get_active_venv())))

    return os.path.isdir(get_envy_path(pkg_path))


def back_up(venv_pkg_path, pkg_path):
    shutil.copytree(venv_pkg_path, get_envy_path(pkg_path))


def get_editor():
    editor = os.environ.get('EDITOR', 'vi')
    if editor == 'vim': editor = 'vi' # vim does not like being launched from python
    return editor


@validate_env
def edit(args):
    pkg_name_given_in_arg = args.path[0].split('/')[0]

    full_package_path = get_venv_full_package_path(pkg_name_given_in_arg)

    split_file_path = args.path[0].split("/")[1:]
    file_path = "/".join(split_file_path)

    if not original_backed_up(args.path[0]):
        print ("backing up {} ".format(full_package_path))
        back_up(full_package_path, args.path[0])
    else:
        print ("backup copy already exists...to restore it before applying new changes run `envy clean {}`".format(pkg_name_given_in_arg))

    editor = get_editor()
    subprocess.call([editor, os.path.join(full_package_path, file_path)], shell = (editor == 'vim'))


@validate_env
@validate_pkg
def sync(args):
    venv_pkg_path = get_venv_full_package_path(args.package[0])
    if not original_backed_up(args.package[0]):
        print ("backing up {} ".format(venv_pkg_path))
        back_up(venv_pkg_path, args.package[0])
    else:
        print ("backup copy already exists...to restore it before applying new changes run `envy clean {}`".format(args.package[0].split('/')[0]))

    try:
        print ("Syncing local changes")
        copytree(args.package[0], venv_pkg_path)
        print ("Local changes synced")
    except Exception as e:
        clean(args)
        raise e


@validate_env
def clean(args):
    if not os.path.isdir(get_envy_path(args.package[0])):
        print ("uh oh..no recorded backup in {}".format(get_envy_path(args.package[0])))
        return

    venv_pkg_path = get_venv_full_package_path(args.package[0])
    print("cleaning applied changes")
    shutil.rmtree(venv_pkg_path)
    print ("restoring original virtualenv state")
    shutil.copytree(get_envy_path(args.package[0]), venv_pkg_path)

    if os.path.isdir(venv_pkg_path):
        # ensure successful copy before removing the backup.
        print ("removing .envie")
        shutil.rmtree(get_envy_path(args.package[0]))


def copytree(src, dst):
    if not os.path.exists(dst):
        os.makedirs(dst)
        shutil.copystat(src, dst)

    if os.path.isfile(src):
        shutil.copy2(src, dst)
        return

    if not os.path.isdir(src):
        src = os.path.expanduser(src)

    for item in os.listdir(src):
        ss = os.path.join(src, item)
        dd = os.path.join(dst, item)
        if os.path.isdir(ss):
            copytree(ss, dd)
        else:
            shutil.copy2(ss, dd)


def prepare_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--version', action='version', version='%(prog)s ' + VERSION)

    subparsers = parser.add_subparsers(dest='command_name')

    parser_edit = subparsers.add_parser('edit', help='edit dependency sourcefile')
    parser_edit.set_defaults(func=edit)
    parser_edit.add_argument('path', nargs=1)

    parser_clean = subparsers.add_parser('clean', help='reset virtualenv to original state')
    parser_clean.set_defaults(func=clean)
    parser_clean.add_argument('package', nargs=1, help='the name of the package to clean')

    parser_sync = subparsers.add_parser('sync', help='sync all files to active virtualenv')
    parser_sync.set_defaults(func=sync)
    parser_sync.add_argument('package', nargs=1, help='the name of changes to sync-- can either be package_name or a path to a file including the package name (i.e. foo/bar.py , where foo is the package)')

    return parser


def main():
    parser = prepare_parser()
    args = parser.parse_args()

    if hasattr(args, 'func'):
        args.func(args)
    else:
        parser.print_help()



VERSION = '0.0.9'
from .application import get_active_venv, get_package_name, get_envy_path,  original_backed_up, get_venv_full_package_path, sync, clean
from .decorators import is_active_venv, in_python_package, validate_env, validate_pkg



import os
import sys

def validate_env(f):
    def wrapper(args):
        if not is_active_venv():
            print("ERR: `envy` must be run from within an active virtualenv!")
            return False
        return f(args)
    return wrapper


def validate_pkg(f):
    def wrapper(args):
        if not in_python_package():
            print("ERR: must be run from within a python package")
            return False
        return f(args)
    return wrapper


def is_active_venv():
    if hasattr(sys, 'real_prefix'):
        return True

    if sys.prefix != getattr(sys, "base_prefix", sys.prefix):
        return True

    return False


def in_python_package():
    return os.path.isfile(os.getcwd() + '/setup.py') or os.path.isfile(os.getcwd() + '/../setup.py')



import sys
import os
import envy
import pkg_resources

from envy import get_active_venv, get_package_name, get_envy_path,  original_backed_up, get_venv_full_package_path

from envy.decorators import is_active_venv, in_python_package

from mock import MagicMock, PropertyMock
from mock import patch

base = os.getcwd()

def setup_test(f):
    @patch('os.getcwd', return_value='{}/tests/testsrc/someuser/src/some_package'.format(base))
    def wrap_patches(*args, **kwargs):
        with patch('envy.application.sys.prefix', "./testsrc/someuser/.virtualenvs/someenv/bin"):
            with patch.dict('os.environ'):
                if 'VIRTUAL_ENV' in os.environ:
                    del os.environ['VIRTUAL_ENV']
                if 'WORKON_HOME' in os.environ:
                    del os.environ['WORKON_HOME']
                return f(*args, **kwargs)

    return wrap_patches

def test_is_active_venv():
    # don't like patching all of sys here but otherwise the tests can have some trouble when run not in a virtualenv
    # (Ideally the tests SHOULD be run from a virtualenv).  THe other option was up to update the application code
    # which I felt would be worse than this..
    with patch('envy.decorators.sys') as mock_sys:
        type(mock_sys).prefix = PropertyMock(return_value="./testsrc/someuser/.virtualenvs/someenv/bin")
        type(mock_sys).real_prefix = PropertyMock(return_value="something/else")
        assert is_active_venv() == True

@setup_test
def test_get_package_name(mock_os):
    assert get_package_name('some_package/__init__.py') == 'some_package'

@setup_test
def test_in_python_package(mock_os):
    assert in_python_package() == True

# would be nice to not have to patch expand user...
@setup_test
@patch('os.path.expanduser', return_value='./testsrc/someuser/.envies/someenv/some_package')
def test_get_envy_path(dummy, mock_os):
    assert get_envy_path('some_package') == './testsrc/someuser/.envies/someenv/some_package'

@setup_test
def test_get_full_package_path(mock_os):
    with patch('envy.application.imp.find_module', return_value=(None,'{}/tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package'.format(base))):
        assert get_venv_full_package_path('some_package') == '{}/tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package'.format(base)

@setup_test
def test_in_python_package_nested_case(mock_os):
    assert in_python_package() == True

@setup_test
def test_get_active_venv_default_case(mock_os):
    assert envy.get_active_venv() == 'someenv'

@patch('os.getcwd', return_value='{}/tests/testsrc/someuser/src/some_package'.format(base))
def test_get_active_venv_non_standard_venv_root(mock_os):
    with patch.dict('os.environ', {'VIRTUAL_ENV': 'some_env_root/someenv'}):
        assert get_active_venv() == 'someenv'

@patch('os.getcwd', return_value='{}/tests/testsrc/someuser/src/some_package'.format(base))
def test_get_active_venv_using_workon_path(mock_os):
    with patch('envy.application.sys.prefix', "./testsrc/someuser/.envs/someenv/bin"):
        with patch.dict('os.environ', {'WORKON_HOME': '.envs'}):
            if 'VIRTUAL_ENV' in os.environ:
                del os.environ['VIRTUAL_ENV']
            assert get_active_venv() == 'someenv'



#!/usr/bin/env python

import os

from setuptools import setup, find_packages

from envy import VERSION

here = os.path.abspath(os.path.dirname(__file__))

required = [
    'future'
]

setup(
    name='envy',
    version=VERSION,
    packages=['envy'],
    url='https://github.com/shaunvxc/envy',
    license='MIT',
    author='Shaun Viguerie',
    author_email='shaunvig114@gmail.com',
    description='safely and easily debug files deep in python virtualenvs',
    entry_points={
        'console_scripts': ['envy = envy.application:main']
    },
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite="tests",
    install_requires=required,
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: End Users/Desktop',
        'Topic :: System :: Shells',
        'Topic :: System :: System Shells',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ]
)



import sys
import os
import envy
import pkg_resources

from envy import get_active_venv, get_package_name, get_envy_path, original_backed_up, get_venv_full_package_path, sync, clean
from envy.decorators import is_active_venv, in_python_package

from mock import MagicMock, PropertyMock
from mock import patch
import argparse

base = os.getcwd()

def setup_test(f):
    """
    All of this patching is annoying, but seems to be the best way to do acceptance testing on envy.
    In the test folder, we have keep a file structure consistent with a common structured of a file system
    with virtual envs.  However, because all of this structure is contained within the test folder of this
    project-- thereby *NOT* real virtual environments-- all of the code responsible for doing this sort of
    thing IRL won't work on this dummy data.  So, we patch it all and let the code responsible for backing
    up copies and moving changes around work with mocked paths.

    The idea is that the acceptance tests + the tests in envy_test.py will suffice.
    """
    def wrap_patches(*args, **kwargs):
        with patch('envy.decorators.is_active_venv', return_value=True):
            with patch('envy.application.get_active_venv', return_value="someenv"):
                with patch('envy.decorators.in_python_package', return_value=True):
                    with patch('envy.application.get_envy_base', return_value="{}/tests/testsrc/someuser/.envies/".format(base)):
                        with patch('envy.application.get_envy_path', return_value="{}/tests/testsrc/someuser/.envies/someenv/some_package".format(base)):
                            with patch('envy.application.get_venv_full_package_path', return_value='{}/tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package'.format(base)):
                                with patch('os.path.expanduser', return_value="{}/tests/testsrc/someuser/src/some_package/some_package".format(base)):
                                    return f(*args, **kwargs)

    return wrap_patches

@patch('os.getcwd', return_value='{}/tests/testsrc/someuser/src/some_package'.format(base))
@setup_test
def test_sync_package_and_clean(mock_os):

    args = argparse.Namespace()
    args.package = ['some_package/test.py']

    sync(args)

    assert os.path.isdir('./tests/testsrc/someuser/.envies/someenv/some_package') == True
    assert os.path.isfile('./tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package/test.py') == True

    clean(args)

    assert os.path.isdir('./tests/testsrc/someuser/.envies/someenv/some_package') == False
    assert os.path.isfile('./tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package/test.py') == False

@patch('os.getcwd', return_value='{}/tests/testsrc/someuser/src/some_package'.format(base))
@setup_test
def test_sync_file_and_clean(mock_os):

    args = argparse.Namespace()
    args.package = ['some_package/test.py']

    sync(args)

    assert os.path.isdir('./tests/testsrc/someuser/.envies/someenv/some_package') == True
    assert os.path.isfile('./tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package/test.py') == True

    clean(args)

    assert os.path.isdir('./tests/testsrc/someuser/.envies/someenv/some_package') == False
    assert os.path.isfile('./tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package/test.py') == False


@patch('os.getcwd', return_value='{}/tests/testsrc/someuser/src/some_package/some_package'.format(base))
@setup_test
def test_sync_file_from_inner_dir_and_clean(mock_os):

    args = argparse.Namespace()
    args.package = ['test.py']

    sync(args)

    assert os.path.isdir('./tests/testsrc/someuser/.envies/someenv/some_package') == True
    assert os.path.isfile('./tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package/test.py') == True

    clean(args)

    assert os.path.isdir('./tests/testsrc/someuser/.envies/someenv/some_package') == False
    assert os.path.isfile('./tests/testsrc/someuser/.virtualenvs/someenv/lib/python2.7/site-packages/some_package/test.py') == False
